@@ -59,14 +59,14 @@
  *  but first make many of these 'static' <<< FIXME!
  */
 void fast_s_large_n(double *X, double *y,
-		    int *nn, int *pp, int *nRes,
+		    int *nn, int *pp, int *nRes, int *max_it_scale,
 		    int *ggroups, int *nn_group,
 		    int *K, int *max_k, double rel_tol, double inv_tol, int *converged,
 		    int *best_r, double *bb, double *rrhoc, int *iipsi,
 		    double *bbeta, double *sscale, int trace_lev, int mts, int ss);
 
 void fast_s(double *X, double *y,
-	    int *nn, int *pp, int *nRes,
+	    int *nn, int *pp, int *nRes, int *max_it_scale,
 	    int *K, int *max_k, double rel_tol, double inv_tol, int *converged,
 	    int *best_r, double *bb, double *rrhoc, int *iipsi,
 	    double *bbeta, double *sscale, int trace_lev, int mts, int ss);
@@ -138,7 +138,7 @@ int refine_fast_s(const double X[], double *wx, const double y[], double *wy,
 		  double *beta_ref, double *scale);
 
 void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
-		   int nResample, double rel_tol, double inv_tol, double *bb,
+		   int nResample, int max_it_scale, double rel_tol, double inv_tol, double *bb,
 		   double *rrhoc, int ipsi, double *sscale, int trace_lev,
 		   double *b1, double *b2, double *t1, double *t2,
 		   double *y_tilde, double *res, double *x1, double *x2,
@@ -146,7 +146,7 @@ void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
 		   double *SC1, double *SC2, double *SC3, double *SC4, int mts, int ss);
 
 void m_s_descent(double *X1, double *X2, double *y,
-		 int n, int p1, int p2, int K_m_s, int max_k,
+		 int n, int p1, int p2, int K_m_s, int max_k, int max_it_scale,
 		 double rel_tol, double *bb, double *rrhoc, int ipsi,
 		 double *sscale, int trace_lev,
 		 double *b1, double *b2, double *t1, double *t2,
@@ -162,7 +162,7 @@ int subsample(const double x[], const double y[], int n, int m,
 	      int sample, int mts, int ss, double tol_inv);
 
 int fast_s_with_memory(double *X, double *y,
-		       int *nn, int *pp, int *nRes,
+		       int *nn, int *pp, int *nRes, int *max_it_scale,
 		       int *K, int *max_k, double rel_tol, double inv_tol,  int trace_lev,
 		       int *best_r, double *bb, double *rrhoc, int *iipsi,
 		       double **best_betas, double *best_scales, int mts, int ss);
@@ -238,13 +238,14 @@ void zero_mat(double **a, int n, int m);
 	}                                                       \
     }
 
-#define SETUP_EQUILIBRATION(_n_, _p_, _X_)	                \
+#define SETUP_EQUILIBRATION(_n_, _p_, _X_, _large_n_)           \
     /* equilibration of matrix _X_                          */  \
     /* solve (Dr X Dc) b = Dr y with beta = Dc b instead of */  \
     /*            X beta = y                                */  \
     /* see Demmel (1997) APPLIED NUMERICAL LINEAR ALGEBRA   */  \
     /*     Section 2.5.2 Equilibration                      */  \
     double *Dr, *Dc, *Xe, rowcnd, colcnd, amax;			\
+    int rowequ = 0 , colequ = 0;                                \
     Dr =        (double *) Calloc(_n_,     double);             \
     Dc =        (double *) Calloc(_p_,     double);             \
     Xe =        (double *) Calloc(_n_*_p_, double);             \
@@ -252,26 +253,30 @@ void zero_mat(double **a, int n, int m);
     F77_CALL(dgeequ)(&_n_, &_p_, Xe, &_n_, Dr, Dc, &rowcnd,	\
     		     &colcnd, &amax, &info);                    \
     if (info) {                                                 \
-	CLEANUP_EQUILIBRATION;                                  \
 	if (info < 0) {                                         \
-	    error("dgeequ: illegal argument in %i. argument", &info); \
+	    CLEANUP_EQUILIBRATION;				\
+	    error("dgeequ: illegal argument in %i. argument", -1 * info); \
 	} else if (info > _n_) {                                \
-            error("dgeequ: column %i is exactly zero.", _n_ - info); \
+	    if (_large_n_) {                                    \
+	        error("Fast S large n strategy failed. Use control parameter 'fast.s.large.n = Inf'."); \
+	    } else {						\
+                error("dgeequ: column %i of the design matrix is exactly zero.", info - _n_); \
+	    }                                                   \
 	} else {                                                \
-	    error("dgeequ: row %i is exactly zero.", info);     \
+	/* FIXME: replace dgeequ by our own version */          \
+	/* that does not treat this as error */                 \
+	    warning("Skipping design matrix equilibration (dgeequ): row %i is exactly zero.", info); \
 	}                                                       \
-    }                                                           \
-    /* scale _X_ */                                             \
-    char equed;                                                 \
-    F77_CALL(dlaqge)(&_n_, &_p_, Xe, &_n_, Dr, Dc, &rowcnd,     \
-                     &colcnd, &amax, &equed);                   \
-    int rowequ = equed == 'B' || equed == 'R';                  \
-    int colequ = equed == 'B' || equed == 'C';/*                   \ */
-    /* Rprintf(" rowequ = %i, colequ = %i, equed = %i\n", rowequ, colequ, equed); \ */
-    /* Rprintf(" Dr = "); disp_vec(Dr, _n_);                       \ */
-    /* Rprintf(" Dc = "); disp_vec(Dc, _p_); */
+    } else {							\
+        /* scale _X_ */                                         \
+        char equed;         					\
+	F77_CALL(dlaqge)(&_n_, &_p_, Xe, &_n_, Dr, Dc, &rowcnd,	\
+			 &colcnd, &amax, &equed);		\
+        rowequ = equed == 'B' || equed == 'R';                  \
+	colequ = equed == 'B' || equed == 'C';                  \
+    }
 
-#define SETUP_SUBSAMPLE(_n_, _p_, _X_)				\
+#define SETUP_SUBSAMPLE(_n_, _p_, _X_, _large_n_)		\
     /* (Pointers to) Arrays - to be allocated */                \
     int *ind_space, *idc, *idr, *pivot;				\
     double *lu, *v;						\
@@ -281,7 +286,7 @@ void zero_mat(double **a, int n, int m);
     pivot =     (int *)    Calloc(_p_-1,   int);                \
     lu =        (double *) Calloc(_p_*_p_, double);             \
     v =         (double *) Calloc(_p_,     double);             \
-    SETUP_EQUILIBRATION(_n_, _p_, _X_);
+    SETUP_EQUILIBRATION(_n_, _p_, _X_, _large_n_);
 
 #define COPY(from, to, len) Memcpy(to, from, len)
 /* This assumes that 'p' is correctly defined, and 'j' can be used in caller: */
@@ -303,7 +308,7 @@ void R_lmrob_S(double *X, double *y, int *n, int *P,
 	       int *best_r, int *Groups, int *N_group,
 	       int *K_s, int *max_k, int *max_it_scale, //double *rel_tol_scale,
 	       double *rel_tol, double *inv_tol, int *converged,
-	       int *trace_lev, int *mts, int *ss)
+	       int *trace_lev, int *mts, int *ss, int *cutoff)
 {
     /* best_r = 't' of Salibian-Barrera_Yohai(2006),
      *	      = no. of best candidates to be iterated further
@@ -312,13 +317,13 @@ void R_lmrob_S(double *X, double *y, int *n, int *P,
     /* Rprintf("R_lmrob_s %d\n", *iipsi); */
 
     if ( *nRes > 0) {
-	if( *n > 2000 )
-	    fast_s_large_n(X, y, n, P, nRes,
+	if( *n > *cutoff )
+	    fast_s_large_n(X, y, n, P, nRes, max_it_scale,
 			   Groups, N_group,
 			   K_s, max_k, *rel_tol, *inv_tol, converged,
 			   best_r, bb, rrhoc, iipsi, beta_s, scale, *trace_lev, *mts, *ss);
 	else
-	    fast_s(X, y, n, P, nRes,
+	    fast_s(X, y, n, P, nRes, max_it_scale,
 		   K_s, max_k, *rel_tol, *inv_tol, converged,
 		   best_r, bb, rrhoc, iipsi, beta_s, scale, *trace_lev, *mts, *ss);
     } else {
@@ -329,7 +334,7 @@ void R_lmrob_S(double *X, double *y, int *n, int *P,
 
 /* Called from R, this function computes an M-S-regression estimator */
 void R_lmrob_M_S(double *X1, double *X2, double *y, double *res,
-		 int *nn, int *pp1, int *pp2, int *nRes,
+		 int *nn, int *pp1, int *pp2, int *nRes, int *max_it_scale,
 		 double *scale, double *b1, double *b2,
 		 double *rho_c, int *ipsi, double *bb,
 		 int *K_m_s, int *max_k, double *rel_tol, double *inv_tol,
@@ -389,7 +394,8 @@ void R_lmrob_M_S(double *X1, double *X2, double *y, double *res,
 
     /* STEP 2: Subsample */
     if (*subsample > 0) {
-	m_s_subsample(X1, y_work, n, p1, p2, *nRes, *rel_tol, *inv_tol, bb,
+	m_s_subsample(X1, y_work, n, p1, p2, *nRes, *max_it_scale, 
+		      *rel_tol, *inv_tol, bb,
 		      rho_c, *ipsi, scale, *trace_lev,
 		      b1, b2, t1, t2, y_tilde, res, x1, x2,
 		      &NIT, &K, &KODE, &SIGMA, &BET0,
@@ -416,8 +422,8 @@ void R_lmrob_M_S(double *X1, double *X2, double *y, double *res,
 
     /* STEP 4: Descent procedure */
     if (*descent > 0) {
-	m_s_descent(X1, X2, y, n, p1, p2, *K_m_s, *max_k, *rel_tol, bb,
-		    rho_c, *ipsi, scale, *trace_lev,
+	m_s_descent(X1, X2, y, n, p1, p2, *K_m_s, *max_k, *max_it_scale, 
+		    *rel_tol, bb, rho_c, *ipsi, scale, *trace_lev,
 		    b1, b2, t1, t2, y_tilde, res, y_work, x1, x2,
 		    &NIT, &K, &KODE, &SIGMA, &BET0, SC1, SC2, SC3, SC4,
 		    converged);
@@ -461,7 +467,7 @@ void R_subsample(const double x[], const double y[], int *n, int *m,
     /*	set the seed */
     GetRNGstate();
 
-    SETUP_EQUILIBRATION(*n, *m, x);
+    SETUP_EQUILIBRATION(*n, *m, x, 0);
 
     *status = subsample(Xe, y, *n, *m, beta, ind_space, idc, idr, lu, v, p,
 			Dr, Dc, rowequ, colequ, *sample, *mts, *ss, *tol_inv);
@@ -1325,7 +1331,7 @@ void zero_mat(double **a, int n, int m)
 /* This function implements the "large n" strategy
  */
 void fast_s_large_n(double *X, double *y,
-		    int *nn, int *pp, int *nRes,
+		    int *nn, int *pp, int *nRes, int *max_it_scale,
 		    int *ggroups, int *nn_group,
 		    int *K, int *max_k, double rel_tol, double inv_tol, int *converged,
 		    int *best_r, double *bb, double *rrhoc, int *iipsi,
@@ -1392,6 +1398,9 @@ void fast_s_large_n(double *X, double *y,
 
     /* get a sample of k indices */
     sample_noreplace(indices, n, sg, ind_space);
+    /* FIXME: define groups using nonsingular subsampling? */
+    /*        would also need to allow observations to be part */
+    /*        of multiple groups at the same time */
     Free(ind_space);
     /* FIXME: Also look at lqs_setup(),
      * -----  and  xr[.,.] "fortran-like" matrix can be used from there!*/
@@ -1410,7 +1419,7 @@ void fast_s_large_n(double *X, double *y,
 	    ysamp[j] = y[indices[ij]];
 	}
 	if(fast_s_with_memory(xsamp, ysamp,
-			      &n_group, pp, nRes, K, max_k, rel_tol, inv_tol,
+			      &n_group, pp, nRes, max_it_scale, K, max_k, rel_tol, inv_tol,
 			      trace_lev, best_r, bb, rrhoc,
 			      iipsi, best_betas + i* *best_r,
 			      best_scales+ i* *best_r, mts, ss)) {
@@ -1471,7 +1480,7 @@ void fast_s_large_n(double *X, double *y,
 	}
 	if ( sum_rho_sc(res, worst_sc, sg, p, rrhoc, ipsi) < b ) {
 	    /* scale will be better */
-	    sc = find_scale(res, b, rrhoc, ipsi, sc, sg, p, /*max_iter*/ 200);
+	    sc = find_scale(res, b, rrhoc, ipsi, sc, sg, p, *max_it_scale);
 	    k2 = pos_worst_scale;
 	    final_best_scales[ k2 ] = sc;
 	    COPY(beta_ref, final_best_betas[k2], p);
@@ -1536,7 +1545,7 @@ void fast_s_large_n(double *X, double *y,
 } /* fast_s_large_n() */
 
 int fast_s_with_memory(double *X, double *y,
-		       int *nn, int *pp, int *nRes,
+		       int *nn, int *pp, int *nRes, int *max_it_scale,
 		       int *K, int *max_k, double rel_tol, double inv_tol,  int trace_lev,
 		       int *best_r, double *bb, double *rrhoc, int *iipsi,
 		       double **best_betas, double *best_scales, int mts, int ss)
@@ -1570,7 +1579,7 @@ int fast_s_with_memory(double *X, double *y,
     int lwork = -1, one = 1, info = 1;
     int pos_worst_scale, sing=0;
 
-    SETUP_SUBSAMPLE(n, p, X);
+    SETUP_SUBSAMPLE(n, p, X, 1);
     INIT_WLS(X, y, n, p);
 
     res	=       (double *) Calloc(n,   double);
@@ -1610,7 +1619,7 @@ int fast_s_with_memory(double *X, double *y,
 
 	if ( sum_rho_sc(res, worst_sc, n, p, rrhoc, ipsi) < b )	{
 	    /* scale will be better */
-	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, /*max_iter*/ 200);
+	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, *max_it_scale);
 	    k = pos_worst_scale;
 	    best_scales[ k ] = sc;
 	    for(j=0; j < p; j++)
@@ -1632,7 +1641,7 @@ int fast_s_with_memory(double *X, double *y,
 } /* fast_s_with_memory() */
 
 void fast_s(double *X, double *y,
-	    int *nn, int *pp, int *nRes,
+	    int *nn, int *pp, int *nRes, int *max_it_scale,
 	    int *K, int *max_k, double rel_tol, double inv_tol, int *converged,
 	    int *best_r, double *bb, double *rrhoc, int *iipsi,
 	    double *bbeta, double *sscale, int trace_lev, int mts, int ss)
@@ -1669,7 +1678,7 @@ void fast_s(double *X, double *y,
     double *wx, *wy, *beta_cand, *beta_ref, *res;
     double **best_betas, *best_scales;
 
-    SETUP_SUBSAMPLE(n, p, X);
+    SETUP_SUBSAMPLE(n, p, X, 0);
 
     res	   = (double *) R_alloc(n, sizeof(double));
     wx     = (double *) R_alloc(n*p, sizeof(double));
@@ -1734,7 +1743,7 @@ void fast_s(double *X, double *y,
 	}
 	if ( sum_rho_sc(res, worst_sc, n, p, rrhoc, ipsi) < b )	{
 	    /* scale will be better */
-	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, /* max_iter*/ 200);
+	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, *max_it_scale);
 	    k = pos_worst_scale;
 	    best_scales[ k ] = sc;
 	    COPY(beta_ref, best_betas[k], p);
@@ -1892,7 +1901,7 @@ int refine_fast_s(const double X[], double *wx, const double y[], double *wy,
 /* Recreates RLFRSTML function found in src/lmrobml.f    */
 /* of the robust package                                 */
 void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
-		   int nResample, double rel_tol, double inv_tol, double *bb,
+		   int nResample, int max_it_scale, double rel_tol, double inv_tol, double *bb,
 		   double *rrhoc, int ipsi, double *sscale, int trace_lev,
 		   double *b1, double *b2, double *t1, double *t2,
 		   double *y_tilde, double *res, double *x1, double *x2,
@@ -1908,7 +1917,7 @@ void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
     if (trace_lev > 1)
 	Rprintf("starting with subsampling procedure...\n");
 
-    SETUP_SUBSAMPLE(n, p2, x2);
+    SETUP_SUBSAMPLE(n, p2, x2, 0);
 
     /*	set the seed */
     GetRNGstate();
@@ -1940,7 +1949,7 @@ void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
 	if (sum_rho_sc(res, *sscale, n, p, rrhoc, ipsi) < b) {
 	    /* scale will be better */
 	    /* STEP 5: Solve for sc */
-	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, /*max_iter*/ 200);
+	    sc = find_scale(res, b, rrhoc, ipsi, sc, n, p, max_it_scale);
 	    if(trace_lev >= 2) {
 		Rprintf("Step %d: new candidate with sc = %.5f\n",i,sc);
 	    }
@@ -1973,7 +1982,7 @@ void m_s_subsample(double *X1, double *y, int n, int p1, int p2,
 
 /* Descent step for M-S algorithm                        */
 void m_s_descent(double *X1, double *X2, double *y,
-		 int n, int p1, int p2, int K_m_s, int max_k,
+		 int n, int p1, int p2, int K_m_s, int max_k, int max_it_scale,
 		 double rel_tol, double *bb, double *rrhoc,  int ipsi,
 		 double *sscale, int trace_lev,
 		 double *b1, double *b2, double *t1, double *t2,
@@ -2033,7 +2042,7 @@ void m_s_descent(double *X1, double *X2, double *y,
 		  *KODE);
 	}
 	/* STEP 3: Compute the scale estimate */
-	sc = find_scale(res2, b, rrhoc, ipsi, sc, n, p, /*max_iter*/ 200);
+	sc = find_scale(res2, b, rrhoc, ipsi, sc, n, p, max_it_scale);
 	/* STEP 4: Check for convergence */
 	/* FIXME: check convergence using scale ? */
 	double del = sqrt(norm_diff2(b1, t1, p1) + norm_diff2(b2, t2, p2));
@@ -2113,7 +2122,7 @@ int subsample(const double x[], const double y[], int n, int m,
                   giving up (Max Try Samples)
        ss:        type of subsampling to be used:
                   0: simple subsampling
-                  1: constrained subsampling
+                  1: nonsingular subsampling
 
        return condition:
              0: success
